use linked_list_allocator::LockedHeap;
use spin::Mutex;

use crate::logger::LOGGER;
use crate::KernelArgs;

use self::memory_map::memory_map;
use self::thunk::ThunkData;
use self::vga::Vga;

#[macro_use]
mod macros;

mod memory_map;
mod panic;
mod thunk;
mod vga;

// Real mode memory allocation, for use with thunk
// 0x500 to 0x7BFF is free
const MEMORY_MAP_ADDR: usize = 0x1380; // 24 bytes, ends at 0x1397
const THUNK_STACK_ADDR: usize = 0x7C00; // Grows downwards
const VGA_ADDR: usize = 0xB8000;

#[global_allocator]
static ALLOCATOR: LockedHeap = LockedHeap::empty();

pub(crate) static VGA: Mutex<Vga> = Mutex::new(unsafe { Vga::new(VGA_ADDR, 80, 25) });

#[no_mangle]
pub unsafe extern "C" fn start(
	_boot_disk: usize,
	thunk10: extern "C" fn(),
	_thunk13: extern "C" fn(),
	thunk15: extern "C" fn(),
	_thunk16: extern "C" fn(),
) -> ! {
	{
		// Make sure we are in mode 3 (80x25 text mode)
		let mut data = ThunkData::new();
		data.eax = 0x03;
		data.with(thunk10);
	}

	{
		// Disable cursor
		let mut data = ThunkData::new();
		data.eax = 0x0100;
		data.ecx = 0x3F00;
		data.with(thunk10);
	}

	// Clear screen
	VGA.lock().clear();

	// Set logger
	LOGGER.init();

	let (heap_start, heap_size) = memory_map(thunk15).expect("No memory for heap");

	ALLOCATOR.lock().init(heap_start as *mut u8, heap_size);
	// let (page_phys, func, args) = crate::run_main(&mut os);

	panic!("kernel");
}
